/* cmdvme
   use /dev/nbbqvio
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "nbbqvio.h"

/* #define DEBUG */

int nbbqvio_fd;
unsigned long nbbqvio_val[2];
unsigned short sval;

int init_nbbqvio(void){
  int try = 0;
  
  while((nbbqvio_fd = open(NBBQVIO_DRV,O_RDWR)) < 0){
    usleep(100000);
    try ++;
    if(try > 100000){
      printf("Can't open %s\n",NBBQVIO_DRV);
      return -1;
    }
  }

  return 0;
}

int release_nbbqvio(void){
  if(nbbqvio_fd < 0){
    return -1;
  }else{
    close(nbbqvio_fd);
  }

  return 0;
}

unsigned short vme_read16(unsigned long addr){
  nbbqvio_val[0] = addr;
  ioctl(nbbqvio_fd,NBBQVIO_READ16,nbbqvio_val);
  sval = nbbqvio_val[1] & 0x0000ffff;
  return sval;
}

unsigned long vme_read32(unsigned long addr){
  nbbqvio_val[0] = addr;
  ioctl(nbbqvio_fd,NBBQVIO_READ32,nbbqvio_val);
  
  return nbbqvio_val[1];
}

int vme_read(unsigned long addr, unsigned short *val){
  *val = vme_read16(addr);
  return 1;
}

int vme_write16(unsigned long addr, unsigned short val){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = val & 0x0000ffff;

  return ioctl(nbbqvio_fd,NBBQVIO_WRITE16,nbbqvio_val);
}

int vme_write32(unsigned long addr, unsigned long val){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = val;
  return ioctl(nbbqvio_fd,NBBQVIO_WRITE32,nbbqvio_val);
}

int vme_write(unsigned long addr, unsigned short *val){
  return vme_write16(addr,*val);
}

int vme_amsr(unsigned long val){
  return ioctl(nbbqvio_fd,NBBQVIO_AMSR,&val);
}


int vread32(unsigned long addr, long *val){
  *val = vme_read32(addr);

  return *val;
}

int vwrite32(unsigned long addr, long *val){
  vme_write32(addr, (unsigned long)*val);

  return 1;
}


int vread16(unsigned long addr, short *val){
  *val = vme_read16(addr);

  return *val;
}

int vwrite16(unsigned long addr, short *val){
  vme_write16(addr, (unsigned short)*val);

  return 1;
}



//------------ i-series ---------------//





unsigned short vme_read16_i(unsigned long addr, int dn){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = dn;
  ioctl(nbbqvio_fd,NBBQVIO_READ16_I,nbbqvio_val);
  sval = nbbqvio_val[1] & 0x0000ffff;
  return sval;
}

unsigned long vme_read32_i(unsigned long addr, int dn){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = dn;
  ioctl(nbbqvio_fd,NBBQVIO_READ32_I,nbbqvio_val);
  
  return nbbqvio_val[1];
}

int vme_read_i(unsigned long addr, unsigned short *val, int dn){
  *val = vme_read16_i(addr, dn);
  return 1;
}

int vme_write16_i(unsigned long addr, unsigned short val, int dn){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = val & 0x0000ffff;
  nbbqvio_val[2] = dn;
  printf("%d!",dn);
  return ioctl(nbbqvio_fd,NBBQVIO_WRITE16_I,nbbqvio_val);
}

int vme_write32_i(unsigned long addr, unsigned long val, int dn){
  nbbqvio_val[0] = addr;
  nbbqvio_val[1] = val;
  nbbqvio_val[2] = dn;
 return ioctl(nbbqvio_fd,NBBQVIO_WRITE32_I,nbbqvio_val);
}

int vme_write_i(unsigned long addr, unsigned short *val, int dn){
  return vme_write16_i(addr,*val, dn);
}

int vme_amsr_i(unsigned long val, int dn){
  nbbqvio_val[0] = val;
  nbbqvio_val[1] = dn;
  printf("%d?",dn);
  return ioctl(nbbqvio_fd,NBBQVIO_AMSR_I,nbbqvio_val);
}


int vread32_i(unsigned long addr, long *val, int dn){
  *val = vme_read32_i(addr, dn);

  return *val;
}

int vwrite32_i(unsigned long addr, long *val, int dn){
  vme_write32_i(addr, (unsigned long)*val, dn);

  return 1;
}


int vread16_i(unsigned long addr, short *val, int dn){
  *val = vme_read16_i(addr, dn);

  return *val;
}

int vwrite16_i(unsigned long addr, short *val, int dn){
  vme_write16_i(addr, (unsigned short)*val, dn);

  return 1;
}

